@@ -85,7 +85,7 @@ module Vagrant
         action_name = environment[:action_name]
 
         # We place a process lock around every action that is called
-        @logger.info("Running action: #{environment[:action_name]} #{callable_id}")
+        @logger.info("Running action: #{action_name} #{callable_id}")
         Util::Busy.busy(int_callback) { callable.call(environment) }
 
         # Return the environment in case there are things in there that
