@@ -90,6 +90,8 @@ module Vagrant
         process.io.stdout = stdout_writer
         process.io.stderr = stderr_writer
         process.duplex = true
+        process.leader = true if @options[:detach]
+        process.detach = true if @options[:detach]
 
         # Special installer-related things
         if Vagrant.in_installer?
@@ -158,6 +160,12 @@ module Vagrant
           raise LaunchError.new(ex.message)
         end
 
+        # If running with the detach option, no need to capture IO or
+        # ensure program exists.
+        if @options[:detach]
+          return
+        end
+
         # Make sure the stdin does not buffer
         process.io.stdin.sync = true
 
@@ -261,7 +269,7 @@ module Vagrant
         # Return an exit status container
         return Result.new(process.exit_code, io_data[:stdout], io_data[:stderr])
       ensure
-        if process && process.alive?
+        if process && process.alive? && !@options[:detach]
           # Make sure no matter what happens, the process exits
           process.stop(2)
         end
