@@ -8,10 +8,12 @@ require "vagrant/config/v2/util"
 require "vagrant/util/platform"
 require "vagrant/util/presence"
 require "vagrant/util/experimental"
+require "vagrant/util/map_command_options"
 
 require File.expand_path("../vm_provisioner", __FILE__)
 require File.expand_path("../vm_subvm", __FILE__)
 require File.expand_path("../disk", __FILE__)
+require File.expand_path("../cloud_init", __FILE__)
 
 module VagrantPlugins
   module Kernel_V2
@@ -21,6 +23,8 @@ module VagrantPlugins
       DEFAULT_VM_NAME = :default
 
       attr_accessor :allowed_synced_folder_types
+      attr_accessor :allow_fstab_modification
+      attr_accessor :allow_hosts_modification
       attr_accessor :base_mac
       attr_accessor :base_address
       attr_accessor :boot_timeout
@@ -46,6 +50,7 @@ module VagrantPlugins
       attr_accessor :usable_port_range
       attr_reader :provisioners
       attr_reader :disks
+      attr_reader :cloud_init_configs
       attr_reader :box_extra_download_options
 
       # This is an experimental feature that isn't public yet.
@@ -55,6 +60,7 @@ module VagrantPlugins
         @logger = Log4r::Logger.new("vagrant::config::vm")
 
         @allowed_synced_folder_types   = UNSET_VALUE
+        @allow_fstab_modification      = UNSET_VALUE
         @base_mac                      = UNSET_VALUE
         @base_address                  = UNSET_VALUE
         @boot_timeout                  = UNSET_VALUE
@@ -72,6 +78,7 @@ module VagrantPlugins
         @box_extra_download_options    = UNSET_VALUE
         @box_url                       = UNSET_VALUE
         @box_version                   = UNSET_VALUE
+        @allow_hosts_modification      = UNSET_VALUE
         @clone                         = UNSET_VALUE
         @communicator                  = UNSET_VALUE
         @graceful_halt_timeout         = UNSET_VALUE
@@ -80,6 +87,7 @@ module VagrantPlugins
         @post_up_message               = UNSET_VALUE
         @provisioners                  = []
         @disks                         = []
+        @cloud_init_configs            = []
         @usable_port_range             = UNSET_VALUE
 
         # Internal state
@@ -152,6 +160,28 @@ module VagrantPlugins
           end
           result.instance_variable_set(:@disks, new_disks)
 
+          # Merge defined cloud_init_configs
+          other_cloud_init_configs = other.instance_variable_get(:@cloud_init_configs)
+          new_cloud_init_configs   = []
+          @cloud_init_configs.each do |p|
+            other_p = other_cloud_init_configs.find { |o| p.id == o.id }
+            if other_p
+              # there is an override. take it.
+              other_p.config = p.config.merge(other_p.config)
+
+              # Remove duplicate disk config from other
+              p = other_p
+              other_cloud_init_configs.delete(other_p)
+            end
+
+            # there is an override, merge it into the
+            new_cloud_init_configs << p.dup
+          end
+          other_cloud_init_configs.each do |p|
+            new_cloud_init_configs << p.dup
+          end
+          result.instance_variable_set(:@cloud_init_configs, new_cloud_init_configs)
+
           # Merge the providers by prepending any configuration blocks we
           # have for providers onto the new configuration.
           other_providers = other.instance_variable_get(:@__providers)
@@ -247,6 +277,11 @@ module VagrantPlugins
 
         options ||= {}
 
+        if options[:nfs]
+          options[:type] = :nfs
+          options.delete(:nfs)
+        end
+
         if options.has_key?(:name)
           synced_folder_name = options.delete(:name)
         else
@@ -370,7 +405,7 @@ module VagrantPlugins
 
           if Vagrant::Util::Experimental.feature_enabled?("dependency_provisioners")
             opts = {before: before, after: after}
-            prov = VagrantConfigProvisioner.new(name, type.to_sym, opts)
+            prov = VagrantConfigProvisioner.new(name, type.to_sym, **opts)
           else
             prov = VagrantConfigProvisioner.new(name, type.to_sym)
           end
@@ -380,7 +415,9 @@ module VagrantPlugins
         prov.preserve_order = !!options.delete(:preserve_order) if \
           options.key?(:preserve_order)
         prov.run = options.delete(:run) if options.key?(:run)
-        prov.add_config(options, &block)
+        prov.communicator_required = options.delete(:communicator_required) if options.key?(:communicator_required)
+
+        prov.add_config(**options, &block)
         nil
       end
 
@@ -435,7 +472,7 @@ module VagrantPlugins
         disk_config.set_options(options)
 
         # Add provider config
-        disk_config.add_provider_config(provider_options, &block)
+        disk_config.add_provider_config(**provider_options, &block)
 
         if !Vagrant::Util::Experimental.feature_enabled?("disks")
           @logger.warn("Disk config defined, but experimental feature is not enabled. To use this feature, enable it with the experimental flag `disks`. Disk will not be added to internal config, and will be ignored.")
@@ -445,6 +482,31 @@ module VagrantPlugins
         @disks << disk_config
       end
 
+      # Stores config options for cloud_init
+      #
+      # @param [Symbol] type
+      # @param [Hash]   options
+      # @param [Block]  block
+      def cloud_init(type=nil, **options, &block)
+        type = type.to_sym if type
+
+        cloud_init_config = VagrantConfigCloudInit.new(type)
+
+        if block_given?
+          block.call(cloud_init_config, VagrantConfigCloudInit)
+        else
+          # config is hash
+          cloud_init_config.set_options(options)
+        end
+
+        if !Vagrant::Util::Experimental.feature_enabled?("cloud_init")
+          @logger.warn("cloud_init config defined, but experimental feature is not enabled. To use this feature, enable it with the experimental flag `cloud_init`. cloud_init config will not be added to internal config, and will be ignored.")
+          return
+        end
+
+        @cloud_init_configs << cloud_init_config
+      end
+
       #-------------------------------------------------------------------
       # Internal methods, don't call these.
       #-------------------------------------------------------------------
@@ -473,6 +535,7 @@ module VagrantPlugins
         @box_version = nil if @box_version == UNSET_VALUE
         @box_download_options = {} if @box_download_options == UNSET_VALUE
         @box_extra_download_options = Vagrant::Util::MapCommandOptions.map_to_command_options(@box_download_options)
+        @allow_hosts_modification = true if @allow_hosts_modification == UNSET_VALUE
         @clone = nil if @clone == UNSET_VALUE
         @communicator = nil if @communicator == UNSET_VALUE
         @graceful_halt_timeout = 60 if @graceful_halt_timeout == UNSET_VALUE
@@ -596,15 +659,6 @@ module VagrantPlugins
 
         current_dir_shared = false
         @__synced_folders.each do |id, options|
-          if options[:nfs]
-            options[:type] = :nfs
-          end
-
-          # Ignore NFS on Windows
-          if options[:type] == :nfs && Vagrant::Util::Platform.windows?
-            options.delete(:type)
-          end
-
           if options[:hostpath]  == '.'
             current_dir_shared = true
           end
@@ -614,6 +668,10 @@ module VagrantPlugins
           d.finalize!
         end
 
+        @cloud_init_configs.each do |c|
+          c.finalize!
+        end
+
         if !current_dir_shared && !@__synced_folders["/vagrant"]
           synced_folder(".", "/vagrant")
         end
@@ -668,6 +726,17 @@ module VagrantPlugins
       def validate(machine, ignore_provider=nil)
         errors = _detected_errors
 
+        if @allow_fstab_modification == UNSET_VALUE
+          machine.synced_folders.types.each do |impl_name|
+            inst = machine.synced_folders.type(impl_name)
+            if inst.capability?(:default_fstab_modification) && inst.capability(:default_fstab_modification) == false
+              @allow_fstab_modification = false
+              break
+            end
+          end
+          @allow_fstab_modification = true if @allow_fstab_modification == UNSET_VALUE
+        end
+
         if !box && !clone && !machine.provider_options[:box_optional]
           errors << I18n.t("vagrant.config.vm.box_missing")
         end
@@ -676,6 +745,10 @@ module VagrantPlugins
           errors << I18n.t("vagrant.config.vm.clone_and_box")
         end
 
+        if box && box.empty?
+          errors << I18n.t("vagrant.config.vm.box_empty", machine_name: machine.name)
+        end
+
         errors << I18n.t("vagrant.config.vm.hostname_invalid_characters", name: machine.name) if \
           @hostname && @hostname !~ /^[a-z0-9][-.a-z0-9]*$/i
 
@@ -725,7 +798,7 @@ module VagrantPlugins
         end
 
         box_download_options.each do |k, v|
-          # If the value is truthy and 
+          # If the value is truthy and
           # if `box_extra_download_options` does not include the key
           # then the conversion to extra download options produced an error
           if v && !box_extra_download_options.include?("--#{k}")
@@ -791,7 +864,17 @@ module VagrantPlugins
         valid_network_types = [:forwarded_port, :private_network, :public_network]
 
         port_range=(1..65535)
+        has_hostname_config = false
         networks.each do |type, options|
+          if options[:hostname]
+            if has_hostname_config
+              errors << I18n.t("vagrant.config.vm.multiple_networks_set_hostname")
+            end
+            if options[:ip] == nil
+              errors << I18n.t("vagrant.config.vm.network_with_hostname_must_set_ip")
+            end
+            has_hostname_config = true
+          end
           if !valid_network_types.include?(type)
             errors << I18n.t("vagrant.config.vm.network_type_invalid",
                             type: type.to_s)
@@ -834,7 +917,7 @@ module VagrantPlugins
         end
 
         # Validate disks
-        # Check if there is more than one primrary disk defined and throw an error
+        # Check if there is more than one primary disk defined and throw an error
         primary_disks = @disks.select { |d| d.primary && d.type == :disk }
         if primary_disks.size > 1
           errors << I18n.t("vagrant.config.vm.multiple_primary_disks_error",
@@ -842,14 +925,29 @@ module VagrantPlugins
         end
 
         disk_names = @disks.map { |d| d.name }
-        duplicate_names = disk_names.detect{ |d| disk_names.count(d) > 1 }
-        if duplicate_names && duplicate_names.size
+        duplicate_names = disk_names.find_all { |d| disk_names.count(d) > 1 }
+        if duplicate_names.any?
           errors << I18n.t("vagrant.config.vm.multiple_disk_names_error",
-                           name: duplicate_names)
+                           name: machine.name,
+                           disk_names: duplicate_names.uniq.join("\n"))
+        end
+
+        disk_files = @disks.map { |d| d.file }
+        duplicate_files = disk_files.find_all { |d| d && disk_files.count(d) > 1 }
+        if duplicate_files.any?
+          errors << I18n.t("vagrant.config.vm.multiple_disk_files_error",
+                           name: machine.name,
+                           disk_files: duplicate_files.uniq.join("\n"))
         end
 
         @disks.each do |d|
           error = d.validate(machine)
+          errors.concat(error) if !error.empty?
+        end
+
+        # Validate clout_init_configs
+        @cloud_init_configs.each do |c|
+          error = c.validate(machine)
           errors.concat error if !error.empty?
         end
 
@@ -927,6 +1025,18 @@ module VagrantPlugins
           end
         end
 
+        if ![TrueClass, FalseClass].include?(@allow_fstab_modification.class)
+          errors["vm"] << I18n.t("vagrant.config.vm.config_type",
+            option: "allow_fstab_modification", given: @allow_fstab_modification.class, required: "Boolean"
+          )
+        end
+
+        if ![TrueClass, FalseClass].include?(@allow_hosts_modification.class)
+          errors["vm"] << I18n.t("vagrant.config.vm.config_type",
+            option: "allow_hosts_modification", given: @allow_hosts_modification.class, required: "Boolean"
+          )
+        end
+
         errors
       end
 
