@@ -54,7 +54,12 @@ module VagrantPlugins
         Cap::ValidateDiskExt
       end
 
-      provider_capability(:virtualbox, :get_default_disk_ext) do
+      provider_capability(:virtualbox, :default_disk_exts) do
+        require_relative "cap/validate_disk_ext"
+        Cap::ValidateDiskExt
+      end
+
+      provider_capability(:virtualbox, :set_default_disk_ext) do
         require_relative "cap/validate_disk_ext"
         Cap::ValidateDiskExt
       end
@@ -63,6 +68,16 @@ module VagrantPlugins
         require_relative "cap"
         Cap
       end
+
+      synced_folder_capability(:virtualbox, "mount_options") do
+        require_relative "cap/mount_options"
+        Cap::MountOptions
+      end
+
+      synced_folder_capability(:virtualbox, "mount_type") do
+        require_relative "cap/mount_options"
+        Cap::MountOptions
+      end
     end
 
     autoload :Action, File.expand_path("../action", __FILE__)
@@ -84,6 +99,8 @@ module VagrantPlugins
 
     module Model
       autoload :ForwardedPort, File.expand_path("../model/forwarded_port", __FILE__)
+      autoload :StorageController, File.expand_path("../model/storage_controller", __FILE__)
+      autoload :StorageControllerArray, File.expand_path("../model/storage_controller_array", __FILE__)
     end
 
     module Util
