@@ -120,7 +120,7 @@ describe VagrantPlugins::CommunicatorSSH::Communicator do
       context "when printing message to the user" do
         before do
           allow(machine).to receive(:ssh_info).
-            and_return(host: '10.1.2.3', port: 22).ordered
+            and_return(host: '10.1.2.3', port: 22)
           allow(communicator).to receive(:connect)
           allow(communicator).to receive(:ready?).and_return(true)
         end
@@ -317,6 +317,15 @@ describe VagrantPlugins::CommunicatorSSH::Communicator do
       end
     end
 
+    context "with no exit code received" do
+      let(:exit_data) { double("exit_data", read_long: nil) }
+
+      it "raises error when exit code is nil" do
+        expect(command_channel).to receive(:send_data).with(/make\n/)
+        expect{ communicator.execute("make") }.to raise_error(Vagrant::Errors::SSHNoExitStatus)
+      end
+    end
+
     context "with garbage content prepended to command output" do
       let(:command_stdout_data) do
         "Line of garbage\nMore garbage\n#{command_garbage_marker}bin\ntmp\n"
@@ -565,6 +574,16 @@ describe VagrantPlugins::CommunicatorSSH::Communicator do
       end
     end
 
+    it "creates remote directory given an empty directory" do
+      file = Dir.mktmpdir
+      begin
+        expect(communicator).to receive(:create_remote_directory).with("/destination/dir/#{ File.basename(file)}/")
+        communicator.upload(file, "/destination/dir")
+      ensure
+        FileUtils.rm_rf(file)
+      end
+    end
+
     it "raises custom error on permission errors" do
       file = Tempfile.new('vagrant-test')
       begin
