@@ -1,6 +1,7 @@
 require File.expand_path("../../../../base", __FILE__)
 
 require Vagrant.source_root.join("plugins/communicators/winssh/communicator")
+require Vagrant.source_root.join("plugins/communicators/winssh/config")
 
 describe VagrantPlugins::CommunicatorWinSSH::Communicator do
   include_context "unit"
@@ -17,12 +18,14 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     )
   end
 
+  let(:shell) { "cmd" }
+
   # SSH configuration information mock
   let(:winssh) do
     double("winssh",
       insert_key: false,
       export_command_template: export_command_template,
-      shell: 'cmd',
+      shell: shell,
       upload_directory: "C:\\Windows\\Temp"
     )
   end
@@ -32,18 +35,21 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
   let(:provider) { double("provider") }
   # UI mock
   let(:ui) { double("ui") }
+  # SSH info mock
+  let(:ssh_info) { double("ssh_info") }
   # Machine mock built with previously defined
   let(:machine) do
     double("machine",
       config: config,
       provider: provider,
-      ui: ui
+      ui: ui,
+      ssh_info: ssh_info
     )
   end
   # Subject instance to test
   let(:communicator){ @communicator ||= described_class.new(machine) }
   # Underlying net-ssh connection mock
-  let(:connection) { double("connection") }
+  let(:connection) { double("connection", open_channel: nil) }
   # Base net-ssh connection channel mock
   let(:channel) { double("channel") }
   # net-ssh connection channel mock for running commands
@@ -60,19 +66,20 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
   let(:command_stdout_data) { '' }
   # Command output returned on stderr
   let(:command_stderr_data) { '' }
-  # Mock for net-ssh scp
-  let(:scp) { double("scp") }
-  # Stub file to match commands
-  let(:ssh_cmd_file){ double("ssh_cmd_file", path: "/dev/null/path") }
+  # Mock for net-ssh sftp
+  let(:sftp) { double("sftp") }
+  # Prevent connection patching by default in tests
+  let(:winssh_patch) { true }
 
   # Setup for commands using the net-ssh connection. This can be reused where needed
   # by providing to `before`
   connection_setup = proc do
+    connection.instance_variable_set(:@winssh_patched, winssh_patch)
     allow(connection).to receive(:logger)
-    allow(connection).to receive(:closed?).and_return false
+    allow(connection).to receive(:closed?).and_return(false)
     allow(connection).to receive(:open_channel).
       and_yield(channel).and_return(channel)
-    allow(channel).to receive(:wait).and_return true
+     allow(channel).to receive(:wait).and_return(true)
     allow(channel).to receive(:close)
     allow(command_channel).to receive(:send_data)
     allow(command_channel).to receive(:eof!)
@@ -88,20 +95,20 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     allow(channel).to receive(:on_request)
     allow(channel).to receive(:on_process)
     allow(channel).to receive(:exec).with(anything).
-      and_yield(command_channel, '').and_return channel
-    expect(command_channel).to receive(:on_request).with('exit-status').
+      and_yield(command_channel, '').and_return(channel)
+    allow(command_channel).to receive(:on_request).with('exit-status').
       and_yield(nil, exit_data)
     # Return mocked net-ssh connection during setup
     allow(communicator).to receive(:retryable).and_return(connection)
-    allow(Tempfile).to receive(:new).with(/vagrant-ssh/).and_return(ssh_cmd_file)
-    allow(ssh_cmd_file).to receive(:puts)
-    allow(ssh_cmd_file).to receive(:close)
-    allow(ssh_cmd_file).to receive(:delete)
-    allow(scp).to receive(:upload!)
-    allow(communicator).to receive(:scp_connect).and_return(true)
+    allow(sftp).to receive(:upload!)
+    allow(communicator).to receive(:sftp_connect).and_return(true)
+    allow(communicator).to receive(:execute).and_call_original
+    allow(communicator).to receive(:execute).
+      with(described_class.const_get(:READY_COMMAND), error_check: false).
+      and_return(0)
   end
 
-  describe ".wait_for_ready" do
+  describe "#wait_for_ready" do
     before(&connection_setup)
     context "with no static config (default scenario)" do
       before do
@@ -124,7 +131,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     end
   end
 
-  describe ".ready?" do
+  describe "#ready?" do
     before(&connection_setup)
     it "returns true if shell test is successful" do
       expect(communicator.ready?).to be_truthy
@@ -132,39 +139,39 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
 
     context "with an invalid shell test" do
       before do
-        expect(exit_data).to receive(:read_long).and_return 1
+        allow(communicator).to receive(:execute).
+          with(described_class.const_get(:READY_COMMAND), error_check: false).
+          and_return(1)
       end
 
       it "returns raises SSHInvalidShell error" do
-        expect{ communicator.ready? }.to raise_error Vagrant::Errors::SSHInvalidShell
+        expect{ communicator.ready? }.to raise_error(Vagrant::Errors::SSHInvalidShell)
       end
     end
   end
 
-  describe ".execute" do
+  describe "#execute" do
     before(&connection_setup)
+
     it "runs valid command and returns successful status code" do
-      expect(ssh_cmd_file).to receive(:puts).with(/dir/)
-      expect(communicator.execute("dir")).to eq(0)
+      expect(communicator.execute("command-to-run", error_check: false)).to eq(0)
     end
 
     it "prepends UUID output to command for garbage removal" do
-      expect(ssh_cmd_file).to receive(:puts).
-        with(/ECHO OFF\nECHO #{command_garbage_marker}\nECHO #{command_garbage_marker}.*/)
-      expect(communicator.execute("dir")).to eq(0)
+      expect(channel).to receive(:exec).
+        with(/Write-Output #{command_garbage_marker};\[Console\]::Error.WriteLine\('#{command_garbage_marker}'\).*/)
+      expect(communicator.execute("command-to-run")).to eq(0)
     end
 
     context "with command returning an error" do
       let(:exit_data) { double("exit_data", read_long: 1) }
 
       it "raises error when exit-code is non-zero" do
-        expect(ssh_cmd_file).to receive(:puts).with(/dir/)
-        expect{ communicator.execute("dir") }.to raise_error(Vagrant::Errors::VagrantError)
+        expect{ communicator.execute("command-to-run") }.to raise_error(Vagrant::Errors::VagrantError)
       end
 
       it "returns exit-code when exit-code is non-zero and error check is disabled" do
-        expect(ssh_cmd_file).to receive(:puts).with(/dir/)
-        expect(communicator.execute("dir", error_check: false)).to eq(1)
+        expect(communicator.execute("command-to-run", error_check: false)).to eq(1)
       end
     end
 
@@ -176,7 +183,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
       it "removes any garbage output prepended to command output" do
         stdout = ''
         expect(
-          communicator.execute("dir") do |type, data|
+          communicator.execute("command-to-run") do |type, data|
             if type == :stdout
               stdout << data
             end
@@ -194,7 +201,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
       it "removes any garbage output prepended to command stderr output" do
         stderr = ''
         expect(
-          communicator.execute("dir") do |type, data|
+          communicator.execute("command-to-run") do |type, data|
             if type == :stderr
               stderr << data
             end
@@ -205,7 +212,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     end
   end
 
-  describe ".test" do
+  describe "#test" do
     before(&connection_setup)
     context "with exit code as zero" do
       it "returns true" do
@@ -215,7 +222,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
 
     context "with exit code as non-zero" do
       before do
-        expect(exit_data).to receive(:read_long).and_return 1
+        expect(exit_data).to receive(:read_long).and_return(1)
       end
 
       it "returns false" do
@@ -224,16 +231,17 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     end
   end
 
-  describe ".upload" do
+  describe "#upload" do
     before do
-      allow(communicator).to receive(:create_remote_directory)
-      expect(communicator).to receive(:scp_connect).and_yield(scp)
+      allow(sftp).to receive(:upload)
+      expect(communicator).to receive(:sftp_connect).and_yield(sftp)
     end
 
     it "uploads a directory if local path is a directory" do
       Dir.mktmpdir('vagrant-test') do |dir|
         FileUtils.touch(File.join(dir, "test-file"))
-        expect(scp).to receive(:upload!).with(an_instance_of(File), /test-file/)
+        expect(sftp).to receive(:mkdir).with(/destination/).exactly(2).times
+        expect(sftp).to receive(:upload!).with(an_instance_of(File), /test-file/)
         communicator.upload(dir, 'C:\destination')
       end
     end
@@ -241,30 +249,21 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     it "uploads a file if local path is a file" do
       file = Tempfile.new('vagrant-test')
       begin
-        expect(scp).to receive(:upload!).with(instance_of(File), 'C:\destination\file')
+        expect(sftp).to receive(:mkdir).with(/destination/)
+        expect(sftp).to receive(:upload!).with(instance_of(File), 'C:/destination/file')
+        expect(Vagrant::Util::Platform).to receive(:unix_windows_path).with('C:\destination\file').
+          and_call_original
         communicator.upload(file.path, 'C:\destination\file')
       ensure
         file.delete
       end
     end
 
-    it "raises custom error on permission errors" do
-      file = Tempfile.new('vagrant-test')
-      begin
-        expect(scp).to receive(:upload!).with(instance_of(File), 'C:\destination\file').
-          and_raise("Permission denied")
-        expect{ communicator.upload(file.path, 'C:\destination\file') }.to(
-          raise_error(Vagrant::Errors::SCPPermissionDenied)
-        )
-      ensure
-        file.delete
-      end
-    end
-
     it "does not raise custom error on non-permission errors" do
       file = Tempfile.new('vagrant-test')
       begin
-        expect(scp).to receive(:upload!).with(instance_of(File), 'C:\destination\file').
+        expect(sftp).to receive(:mkdir).with(/destination/)
+        expect(sftp).to receive(:upload!).with(instance_of(File), 'C:/destination/file').
           and_raise("Some other error")
         expect{ communicator.upload(file.path, 'C:\destination\file') }.to raise_error(RuntimeError)
       ensure
@@ -273,18 +272,18 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
     end
   end
 
-  describe ".download" do
+  describe "#download" do
     before do
-      expect(communicator).to receive(:scp_connect).and_yield(scp)
+      expect(communicator).to receive(:sftp_connect).and_yield(sftp)
     end
 
-    it "calls scp to download file" do
-      expect(scp).to receive(:download!).with('/path/from', 'C:\path\to')
+    it "calls sftp to download file" do
+      expect(sftp).to receive(:download!).with('/path/from', 'C:\path\to')
       communicator.download('/path/from', 'C:\path\to')
     end
   end
 
-  describe ".connect" do
+  describe "#connect" do
 
     it "cannot be called directly" do
       expect{ communicator.connect }.to raise_error(NoMethodError)
@@ -309,7 +308,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           nil, nil, hash_including(
             keys_only: true
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -318,7 +317,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           nil, nil, hash_including(
             verify_host_key: false
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -327,7 +326,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           nil, nil, hash_excluding(
             keys: anything
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -336,7 +335,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           nil, nil, hash_including(
             auth_methods: ["none", "hostbased"]
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
     end
@@ -360,7 +359,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           nil, nil, hash_including(
             keys_only: false
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -369,7 +368,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           nil, nil, hash_including(
             verify_host_key: true
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
     end
@@ -389,12 +388,14 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
       end
 
       it "specifies configured host" do
-        expect(Net::SSH).to receive(:start).with("127.0.0.1", anything, anything)
+        expect(Net::SSH).to receive(:start).with("127.0.0.1", anything, anything).
+          and_return(connection)
         communicator.send(:connect)
       end
 
       it "has port defined" do
-        expect(Net::SSH).to receive(:start).with("127.0.0.1", anything, hash_including(port: 2222))
+        expect(Net::SSH).to receive(:start).with("127.0.0.1", anything, hash_including(port: 2222)).
+          and_return(connection)
         communicator.send(:connect)
       end
     end
@@ -417,7 +418,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             keys: ["/priv/key/path"]
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -426,13 +427,12 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             auth_methods: ["none", "hostbased", "publickey"]
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
     end
 
     context "with username and password configured" do
-
       before do
         expect(machine).to receive(:ssh_info).and_return(
           host: '127.0.0.1',
@@ -446,7 +446,8 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
       end
 
       it "has username defined" do
-        expect(Net::SSH).to receive(:start).with(anything, 'vagrant', anything).and_return(true)
+        expect(Net::SSH).to receive(:start).with(anything, 'vagrant', anything).
+          and_return(connection)
         communicator.send(:connect)
       end
 
@@ -455,7 +456,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             password: 'vagrant'
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -464,7 +465,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             auth_methods: ["none", "hostbased", "password"]
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
     end
@@ -488,7 +489,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             password: 'vagrant'
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -497,7 +498,7 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             keys: ["/priv/key/path"]
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
 
@@ -506,19 +507,46 @@ describe VagrantPlugins::CommunicatorWinSSH::Communicator do
           anything, anything, hash_including(
             auth_methods: ["none", "hostbased", "publickey", "password"]
           )
-        ).and_return(true)
+        ).and_return(connection)
         communicator.send(:connect)
       end
     end
+
+    context "when not patched for winssh" do
+      let(:winssh_patch) { false }
+
+      before(&connection_setup)
+
+      it "should patch the connection instance on first request" do
+        expect(connection).to receive(:define_singleton_method)
+        communicator.send(:connect)
+      end
+
+      it "should force powershell on exec" do
+        expect(channel).to receive(:exec).with(/powershell/).and_return(channel)
+        communicator.execute("test", error_check: false)
+      end
+    end
   end
 
-  describe ".generate_environment_export" do
+  describe "#generate_environment_export" do
+    let(:winssh) do
+      @c ||= VagrantPlugins::CommunicatorWinSSH::Config.new
+      @c.finalize!
+      @c
+    end
+
     it "should generate bourne shell compatible export" do
-      expect(communicator.send(:generate_environment_export, "TEST", "value")).to eq("export TEST=\"value\"\n")
+      expect(communicator.send(:generate_environment_export, "TEST", "value")).to eq("$env:TEST=\"value\"\n")
     end
 
     context "with custom template defined" do
-      let(:export_command_template){ "setenv %ENV_KEY% %ENV_VALUE%" }
+      let(:winssh) do
+        @c ||= VagrantPlugins::CommunicatorWinSSH::Config.new
+        @c.export_command_template = "setenv %ENV_KEY% %ENV_VALUE%"
+        @c.finalize!
+        @c
+      end
 
       it "should generate custom export based on template" do
         expect(communicator.send(:generate_environment_export, "TEST", "value")).to eq("setenv TEST value\n")
