@@ -67,7 +67,7 @@ describe "VagrantPlugins::GuestDebian::Cap::ConfigureNetworks" do
     before do
       allow(comm).to receive(:test).with("nmcli -t d show eth1").and_return(false)
       allow(comm).to receive(:test).with("nmcli -t d show eth2").and_return(false)
-      allow(comm).to receive(:test).with("ps -o comm= 1 | grep systemd").and_return(false)
+      allow(comm).to receive(:test).with("ps -o comm= 1 | grep systemd", {sudo: true}).and_return(false)
       allow(comm).to receive(:test).with("systemctl -q is-active systemd-networkd.service", anything).and_return(false)
       allow(comm).to receive(:test).with("command -v netplan").and_return(false)
     end
@@ -85,7 +85,7 @@ describe "VagrantPlugins::GuestDebian::Cap::ConfigureNetworks" do
 
     context "with systemd" do
       before do
-        expect(comm).to receive(:test).with("ps -o comm= 1 | grep systemd").and_return(true)
+        expect(comm).to receive(:test).with("ps -o comm= 1 | grep systemd", {sudo: true}).and_return(true)
         allow(comm).to receive(:test).with("command -v netplan").and_return(false)
       end
 
@@ -140,6 +140,8 @@ describe "VagrantPlugins::GuestDebian::Cap::ConfigureNetworks" do
         let(:networkd_yml) { "---\nnetwork:\n  version: 2\n  renderer: networkd\n  ethernets:\n    eth1:\n      dhcp4: true\n    eth2:\n      addresses:\n      - 33.33.33.10/16\n      gateway4: 33.33.0.1\n" }
 
         it "uses NetworkManager if detected on device" do
+          allow(cap).to receive(:systemd_networkd?).and_return(false)
+          allow(cap).to receive(:nmcli?).and_return(true)
           allow(cap).to receive(:nm_controlled?).and_return(true)
           allow(comm).to receive(:test).with("nmcli -t d show eth1").and_return(true)
           allow(comm).to receive(:test).with("nmcli -t d show eth2").and_return(true)
@@ -156,7 +158,18 @@ describe "VagrantPlugins::GuestDebian::Cap::ConfigureNetworks" do
           expect(comm.received_commands[0]).to match("netplan apply")
         end
 
+        it "raises and error if NetworkManager is detected on device but nmcli is not installed" do
+          allow(cap).to receive(:systemd_networkd?).and_return(true)
+          allow(cap).to receive(:nmcli?).and_return(false)
+          allow(cap).to receive(:nm_controlled?).and_return(true)
+          allow(comm).to receive(:test).with("nmcli -t d show eth1").and_return(true)
+          allow(comm).to receive(:test).with("nmcli -t d show eth2").and_return(true)
+
+          expect { cap.configure_networks(machine, [network_0, network_1]) }.to raise_error(Vagrant::Errors::NetworkManagerNotInstalled)
+        end
+
         it "creates and starts the networks for systemd with netplan" do
+          allow(cap).to receive(:systemd_networkd?).and_return(true)
           expect(cap).to receive(:upload_tmp_file).with(comm, networkd_yml)
             .and_return("/tmp/vagrant-network-entry.1234")
 
