@@ -32,23 +32,23 @@ describe VagrantPlugins::SyncedFolderRSync::RsyncHelper do
     let(:path) { "/foo/bar" }
 
     it "converts a directory match" do
-      expect(described_class.exclude_to_regexp(path, "foo/")).
-        to eq(/^#{Regexp.escape(path)}\/.*foo\//)
+      expect(described_class.exclude_to_regexp("foo/")).
+        to eq(/^foo\/.[^\/]*/)
     end
 
     it "converts the start anchor" do
-      expect(described_class.exclude_to_regexp(path, "/foo")).
-        to eq(/^\/foo\/bar\/foo/)
+      expect(described_class.exclude_to_regexp("/foo")).
+        to eq(/^foo\//)
     end
 
     it "converts the **" do
-      expect(described_class.exclude_to_regexp(path, "fo**o")).
-        to eq(/^#{Regexp.escape(path)}\/.*fo.*o/)
+      expect(described_class.exclude_to_regexp("fo**o")).
+        to eq(/^fo.*o\/.[^\/]*/)
     end
 
     it "converts the *" do
-      expect(described_class.exclude_to_regexp(path, "fo*o")).
-        to eq(/^#{Regexp.escape(path)}\/.*fo[^\/]*o/)
+      expect(described_class.exclude_to_regexp("fo*o")).
+        to eq(/^fo[^\/]*o\/.[^\/]*/)
     end
   end
 
@@ -292,6 +292,20 @@ describe VagrantPlugins::SyncedFolderRSync::RsyncHelper do
           expect(args[9]).to include("ControlPath=/tmp/vagrant-rsync-12345")
         }.and_return(result)
 
+        expect(FileUtils).to receive(:remove_entry_secure).with("/tmp/vagrant-rsync-12345", true).and_return(true)
+        subject.rsync_single(machine, ssh_info, opts)
+      end
+
+      it "does not create tmp dir on windows platforms" do
+        allow(Vagrant::Util::Platform).to receive(:windows?).and_return(true)
+        allow(Dir).to receive(:mktmpdir).with("vagrant-rsync-").
+          and_return("/tmp/vagrant-rsync-12345")
+
+        expect(Vagrant::Util::Subprocess).to receive(:execute).with(any_args) { |*args|
+          expect(args).not_to include("ControlPath=/tmp/vagrant-rsync-12345")
+        }.and_return(result)
+
+        expect(FileUtils).not_to receive(:remove_entry_secure).with("/tmp/vagrant-rsync-12345", true)
         subject.rsync_single(machine, ssh_info, opts)
       end
     end
