@@ -91,7 +91,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       env[:box_name] = "foo"
       env[:box_url] = box_path.to_s
 
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo")
         expect(version).to eq("0")
@@ -113,7 +113,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         box_path.to_s,
       ]
 
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo")
         expect(version).to eq("0")
@@ -132,7 +132,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         env[:box_name] = "foo"
         env[:box_url] = "http://127.0.0.1:#{port}/#{box_path.basename}"
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(checksum(path)).to eq(checksum(box_path))
           expect(name).to eq("foo")
           expect(version).to eq("0")
@@ -152,7 +152,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         env[:box_name] = "foo"
         env[:box_url] = "ftp://127.0.0.1:#{port}/#{box_path.basename}"
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(checksum(path)).to eq(checksum(box_path))
           expect(name).to eq("foo")
           expect(version).to eq("0")
@@ -218,7 +218,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         env[:box_checksum_type] = ""
 
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(checksum(path)).to eq(checksum(box_path))
           expect(name).to eq("foo")
           expect(version).to eq("0")
@@ -291,7 +291,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       env[:box_provider] = "virtualbox"
 
       allow(box_collection).to receive(:find).and_return(box)
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo")
         expect(version).to eq("0")
@@ -311,7 +311,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         box_url_name = "http://127.0.0.1:#{port}/#{box_path.basename}"
         env[:box_name] = box_url_name
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq(box_url_name)
           expect(version).to eq("0")
           expect(opts[:metadata_url]).to be_nil
@@ -333,7 +333,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         box_url_name = "box name with spaces"
         env[:box_name] = box_url_name
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq(box_url_name)
           expect(version).to eq("0")
           expect(opts[:metadata_url]).to be_nil
@@ -356,7 +356,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
           env[:box_name] = "foo"
           env[:box_url] = "http://#{username}:#{password}@127.0.0.1:#{port}/#{box_path.basename}"
 
-          expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+          expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
             expect(checksum(path)).to eq(checksum(box_path))
             expect(name).to eq("foo")
             expect(version).to eq("0")
@@ -404,7 +404,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       with_web_server(md_path) do |port|
         env[:box_url] = "http://127.0.0.1:#{port}/#{md_path.basename}"
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq("foo/bar")
           expect(version).to eq("0.7")
           expect(checksum(path)).to eq(checksum(box_path))
@@ -449,7 +449,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       with_web_server(md_path, **opts) do |port|
         env[:box_url] = "http://127.0.0.1:#{port}/#{md_path.basename}"
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq("foo/bar")
           expect(version).to eq("0.7")
           expect(checksum(path)).to eq(checksum(box_path))
@@ -494,7 +494,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         url = "http://127.0.0.1:#{port}"
         env[:box_url] = "mitchellh/precise64.json"
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq("mitchellh/precise64")
           expect(version).to eq("0.7")
           expect(checksum(path)).to eq(checksum(box_path))
@@ -545,7 +545,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         env[:box_url] = "mitchellh/precise64.json"
         env[:box_server_url] = url
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq("mitchellh/precise64")
           expect(version).to eq("0.7")
           expect(checksum(path)).to eq(checksum(box_path))
@@ -595,8 +595,10 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
         env[:box_url] = "foo"
 
         env[:hook] = double("hook")
-        allow(env[:hook]).to receive(:call) do |name, opts|
-          expect(name).to eq(:authenticate_box_url)
+
+        expect(env[:hook]).to receive(:call).with(:authenticate_box_downloader, any_args).at_least(:once)
+
+        allow(env[:hook]).to receive(:call).with(:authenticate_box_url, any_args).at_least(:once) do |name, opts|
           if opts[:box_urls] == ["foo"]
             next { box_urls: [real_url] }
           elsif opts[:box_urls] == ["bar"]
@@ -606,7 +608,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
           end
         end
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq("foo/bar")
           expect(version).to eq("0.7")
           expect(checksum(path)).to eq(checksum(box_path))
@@ -651,7 +653,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       with_web_server(md_path) do |port|
         env[:box_url] = "http://127.0.0.1:#{port}/#{md_path.basename}"
 
-        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+        expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
           expect(name).to eq("foo/bar")
           expect(version).to eq("0.7")
           expect(checksum(path)).to eq(checksum(box_path))
@@ -795,7 +797,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       end
 
       env[:box_url] = tf.path
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.7")
@@ -839,7 +841,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
 
       env[:box_url] = tf.path
       env[:box_provider] = "vmware"
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.7")
@@ -888,7 +890,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
 
       env[:box_url] = tf.path
       env[:box_provider] = "vmware"
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.7")
@@ -928,7 +930,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
 
       env[:box_url] = tf.path
       env[:box_version] = "~> 0.1"
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.5")
@@ -973,7 +975,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       env[:box_url] = tf.path
       env[:box_provider] = "vmware"
       env[:box_version] = "~> 0.1"
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.5")
@@ -1021,7 +1023,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
 
       env[:box_url] = tf.path
       env[:box_provider] = ["virtualbox", "vmware"]
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.7")
@@ -1069,7 +1071,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
 
       expect(env[:ui]).to receive(:ask).and_return("1")
 
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.7")
@@ -1245,7 +1247,7 @@ describe Vagrant::Action::Builtin::BoxAdd, :skip_windows, :bsdtar do
       env[:box_force] = true
       env[:box_url] = tf.path
       allow(box_collection).to receive(:find).and_return(box)
-      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, **opts|
+      expect(box_collection).to receive(:add).with(any_args) { |path, name, version, opts|
         expect(checksum(path)).to eq(checksum(box_path))
         expect(name).to eq("foo/bar")
         expect(version).to eq("0.7")
