@@ -14,7 +14,7 @@ describe Vagrant::UI::Basic do
     end
 
     it "outputs using `puts` by default" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:printer]).to eq(:puts)
         true
       }
@@ -23,7 +23,7 @@ describe Vagrant::UI::Basic do
     end
 
     it "outputs using `print` if new_line is false" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:printer]).to eq(:print)
         true
       }
@@ -32,7 +32,7 @@ describe Vagrant::UI::Basic do
     end
 
     it "outputs using `print` if new_line is false" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:printer]).to eq(:print)
         true
       }
@@ -44,7 +44,7 @@ describe Vagrant::UI::Basic do
       stdout = StringIO.new
       subject.stdout = stdout
 
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:io]).to be(stdout)
         true
       }
@@ -60,7 +60,7 @@ describe Vagrant::UI::Basic do
       stderr = StringIO.new
       subject.stderr = stderr
 
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(opts[:io]).to be(stderr)
         true
       }
@@ -81,7 +81,7 @@ describe Vagrant::UI::Basic do
 
   context "#detail" do
     it "outputs details" do
-      expect(subject).to receive(:safe_puts).with(any_args) { |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) { |message, opts|
         expect(message).to eq("foo")
         true
       }
@@ -104,7 +104,7 @@ describe Vagrant::UI::Basic do
     before{ Vagrant::Util::CredentialScrubber.sensitive(password) }
 
     it "should remove sensitive information from the output" do
-      expect(subject).to receive(:safe_puts).with(any_args) do |message, **opts|
+      expect(subject).to receive(:safe_puts).with(any_args) do |message, opts|
         expect(message).not_to include(password)
       end
       subject.detail(output)
